SystemOrganization addCategory: #'Pier-SW2-AdvancedSearch'!

!PRStructure methodsFor: '*pier-sw2-advancedsearch' stamp: 'JCB 4/2/2009 22:56'!
fullAdvancedSearch
	"Answer an advanced search visitor starting at the receiver."

	^ PRCriteriaSearch on: self! !

PRSearchWidget subclass: #PRAdvancedSearchWidget
	instanceVariableNames: 'searchCriterions isStartingSearch report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-SW2-AdvancedSearch'!

!PRAdvancedSearchWidget commentStamp: 'JCB 7/16/2009 20:37' prior: 0!
A PRAdvancedSearchWidget is widget for doing more complex searches than single phrases.  It is meant for large wikis that can't be easily searched (that is why it isn't modelled after the ajax search).

Instance Variables
	isStartingSearch:		<Boolean>
	report:		<WATableReport>
	searchCriterions:		<OrderedCollection of WAMatchCriteria>

isStartingSearch
	- Should be true only when search is in progress.

report
	- Display of what was found, needed for #children

searchCriterions
	- Criteria for searching

generateReport:
	- This creates the report that is displayed.!

!PRAdvancedSearchWidget class methodsFor: 'accessing' stamp: 'JCB 3/31/2009 21:38'!
label
	^ 'Advanced Search'! !

!PRAdvancedSearchWidget methodsFor: 'accessing' stamp: 'JCB 7/18/2009 15:32'!
addCriterion
	"Call the accessor to make sure its initialized."
	self searchCriterions size > 1000000 ifTrue: [Transcript cr; show: 'Expect performance problems with searching.'].
	searchCriterions add: PRMatchCriteria new! !

!PRAdvancedSearchWidget methodsFor: 'accessing' stamp: 'JCB 7/15/2009 22:28'!
children
	^ super children, (Array with: self report)! !

!PRAdvancedSearchWidget methodsFor: 'searching' stamp: 'JCB 5/26/2009 20:38'!
findItems
	^ self isStartingSearch
		ifTrue: [isStartingSearch := false.
			(self context structure fullAdvancedSearch criterions: self searchCriterions) contents]
		ifFalse: [#()]! !

!PRAdvancedSearchWidget methodsFor: 'rendering' stamp: 'JCB 7/17/2009 08:24'!
generateReport: pages 
	self report rows: (self sort: pages);		
		columns: (OrderedCollection
				with: (WAReportColumn
						renderBlock: [:each :html | html anchor goto: (self context structure: each); with: each absolutePath]
						title: 'Name')
				with: (WAReportColumn
						renderBlock: [:each :html || histories | histories := self context kernel persistency historyFor: each.
							histories isEmpty ifTrue: [html text: 'N/A'] 
								ifFalse: [html text: histories last command timestamp]]
						title: 'Last Date Modified'));
		 rowColors: #(#white #lightblue );
		 rowPeriod: 1;
		 yourself! !

!PRAdvancedSearchWidget methodsFor: 'accessing' stamp: 'JCB 4/5/2009 13:02'!
isStartingSearch
	isStartingSearch isNil ifTrue: [isStartingSearch := false].
	^ isStartingSearch! !

!PRAdvancedSearchWidget methodsFor: 'rendering' stamp: 'JCB 5/21/2009 06:34'!
renderContentOn: html 
	html
		form: [self searchCriterions
				do: [:e | self renderCriterion: e on: html].
			html break.
			html submitButton
				callback: [self addCriterion];
				with: 'Add More Criteria'.
			html space.
			html submitButton
				callback: [isStartingSearch := true.
					items := nil];
				 with: 'search'].
	self items isEmpty
		ifFalse: [self renderReportOn: html]! !

!PRAdvancedSearchWidget methodsFor: 'rendering' stamp: 'JCB 4/5/2009 13:50'!
renderCriterion: aCriteria on: html 
	aCriteria = self searchCriterions first
		ifTrue: [html text: 'Search for ']
		ifFalse: [html break.
			html select list: #(#and #or );
				 selected: aCriteria combinationType;
				labels: [:each | each asString];
				callback: [:value | aCriteria combinationType: value]].
	html select list: #(true false );
		 selected: aCriteria isNegated;
		labels: [:each | each
				ifTrue: ['not like']
				ifFalse: ['like']];
		callback: [:value | aCriteria isNegated: value].
	html textInput value: aCriteria textToMatch;
		callback: [:value | aCriteria textToMatch: value].
	html text: ' found in '.
	html select list: #('*' 'Title' 'Contents' );
		 selected: aCriteria fieldName;
		labels: [:each | each = '*'
				ifTrue: ['any']
				ifFalse: [each]];
		callback: [:value | aCriteria fieldName: value].
	html text: ' using '.
	html select list: #(true false );
		 selected: aCriteria isCaseSensitive;
		labels: [:each | each
				ifTrue: ['case sensitive']
				ifFalse: ['case insensitive']];
		callback: [:value | aCriteria isCaseSensitive: value].
	self searchCriterions size > 1
		ifTrue: [html space.
			html submitButton
				callback: [searchCriterions remove: aCriteria];
				 with: 'remove this row'.
		self searchCriterions first combinationType: #and]! !

!PRAdvancedSearchWidget methodsFor: 'rendering' stamp: 'JCB 5/27/2009 11:24'!
renderReportOn: html 
	"modelled the below after PBPostRelated>>renderBlogOn:"
	self generateReport: self items.
	html render: report
! !

!PRAdvancedSearchWidget methodsFor: 'accessing' stamp: 'JCB 4/6/2009 20:25'!
report
	report isNil
		ifTrue: [report := WATableReport new].
	^ report! !

!PRAdvancedSearchWidget methodsFor: 'accessing' stamp: 'JCB 4/4/2009 14:09'!
searchCriterions
	searchCriterions isNil
		ifTrue: [searchCriterions := OrderedCollection
						with: PRMatchCriteria new].
	^ searchCriterions! !

!PRAdvancedSearchWidget methodsFor: 'sorting' stamp: 'JCB 7/17/2009 08:25'!
sort: structures
	^ structures asSortedCollection: [:a :b || bHistory aHistory |  
		bHistory := self context kernel persistency historyFor: b.
		bHistory isEmpty ifTrue: [true] ifFalse: [aHistory := self context kernel persistency historyFor: a.
			aHistory isEmpty ifTrue: [false] ifFalse: [aHistory last command timestamp > bHistory last command timestamp]]]! !

Object subclass: #PRMatchCriteria
	instanceVariableNames: 'fieldName textToMatch isNegated combinationType isCaseSensitive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-SW2-AdvancedSearch'!

!PRMatchCriteria commentStamp: 'JCB 3/29/2009 22:39' prior: 0!
A PRMatchCriteria is class for matching search criteria against specific pier pages.

Instance Variables
	fieldName		<String>
	textToMatch:		<String>
	isNegated:		<Boolean>
	combinationType:		<Symbol>
	isCaseSensitive		<Boolean>

fieldName - like '*' or 'Title' - Taken from the label of a MA*Description

textToMatch - String to match
	
combinationType
	- Values are either #and or #or (xor seems to complicated to explain to users)

isNegated
	- True if the user wants pages that don't match the criteria.

isCaseSensitive
	- True if the search is case sensitive.
	!

!PRMatchCriteria class methodsFor: 'instance creation' stamp: 'JCB 3/29/2009 22:43'!
newFieldName: aFieldString textToMatch: aMatchString isNegated: negated combinationType: aSymbol isCaseSensitive: checkCase 
	^ self new fieldName: aFieldString;
		 textToMatch: aMatchString;
		 isNegated: negated;
		 combinationType: aSymbol;
		 isCaseSensitive: checkCase;
		 yourself! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 21:46'!
combinationType
	^ combinationType! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:07'!
combinationType: aSymbol
	combinationType := aSymbol! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:17'!
fieldName
	^ fieldName! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:18'!
fieldName: aString
	fieldName := aString! !

!PRMatchCriteria methodsFor: 'initialize-release' stamp: 'JCB 4/5/2009 10:10'!
initialize
	^ self fieldName: '*';
		 textToMatch: '';
		 isNegated: false;
		 combinationType: #and;
		 isCaseSensitive: false;
		 yourself! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:33'!
isCaseSensitive
	^ isCaseSensitive! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:33'!
isCaseSensitive: aBoolean
	isCaseSensitive := aBoolean! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 21:46'!
isNegated
	^ isNegated ! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:07'!
isNegated: aBoolean
	isNegated := aBoolean! !

!PRMatchCriteria methodsFor: 'matching' stamp: 'JCB 3/30/2009 22:22'!
matchAgainst: aStructure 
	aStructure description
		do: [:aDescription | (aDescription isSearchable and: [(self fieldName match: aDescription label)])
				ifTrue: [((aDescription
							toString: (aStructure readUsing: aDescription))
							includesSubstring: self textToMatch
							caseSensitive: self isCaseSensitive)
						ifTrue: [^ self isNegated not]]].
	^ self isNegated! !

!PRMatchCriteria methodsFor: 'as yet unclassified' stamp: 'JCB 4/1/2009 23:00'!
printOn: aStream 
	aStream nextPutAll: self combinationType.
	aStream
		nextPutAll: (self isNegated
				ifTrue: [' not ']
				ifFalse: [' ']);
		 nextPut: $";
		 nextPutAll: self textToMatch;
		 nextPutAll: '" in ';
		 nextPutAll: self fieldName;
		nextPutAll: (self isCaseSensitive
				ifTrue: [' match case']
				ifFalse: [' ignoring case'])! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 21:44'!
textToMatch
	^ textToMatch ! !

!PRMatchCriteria methodsFor: 'accessing' stamp: 'JCB 3/29/2009 22:08'!
textToMatch: aString
	textToMatch := aString! !

PRFullTextSearch subclass: #PRCriteriaSearch
	instanceVariableNames: 'criterions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-SW2-AdvancedSearch'!

!PRCriteriaSearch methodsFor: 'as yet unclassified' stamp: 'JCB 4/2/2009 22:02'!
criterions
	criterions isNil
		ifTrue: [self criterions: #()].
	^ criterions! !

!PRCriteriaSearch methodsFor: 'as yet unclassified' stamp: 'JCB 4/2/2009 22:01'!
criterions: aPRMatchCriteriaOrderedCollection
	criterions := aPRMatchCriteriaOrderedCollection! !

!PRCriteriaSearch methodsFor: 'as yet unclassified' stamp: 'JCB 4/2/2009 23:04'!
findItems	
	^ (self context structure fullAdvancedSearch criterions: self criterions) contents! !

!PRCriteriaSearch methodsFor: 'as yet unclassified' stamp: 'JCB 4/5/2009 13:26'!
visitStructure: aStructure 
	(self criterions
			inject: true
			into: [:b :c | ((b
							and: [c combinationType = #or])
						or: [b
								and: [c matchAgainst: aStructure]])
					or: [b not
							and: [c combinationType = #or
									and: [c matchAgainst: aStructure]]]])
		ifTrue: [block value: aStructure]! !

PRObjectTest subclass: #PRAdvancedSearchWditgetTest
	instanceVariableNames: 'advancedSearch pageEdit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-SW2-AdvancedSearch'!

!PRAdvancedSearchWditgetTest commentStamp: 'JCB 7/18/2009 15:29' prior: 0!
PRAdvancedSearchWditgetTest is for testing the advanced search widget.

Instance Variables
	advancedSearch:		<PRAdvancedSearchWidget>
	pageEdit:		<PREditCommand>

When null persistency is the persistency, no page has any history, so the sort won't put the most recently edited page first (but at least it won't fail).  If history persistency is used, it should order the most recently edited page first.!

!PRAdvancedSearchWditgetTest class methodsFor: 'testing' stamp: 'JCB 7/18/2009 15:30'!
isAbstract
	^ false! !

!PRAdvancedSearchWditgetTest methodsFor: 'private' stamp: 'JCB 7/18/2009 10:51'!
actualClass
	^ PRAdvancedSearchWidget ! !

!PRAdvancedSearchWditgetTest methodsFor: 'private' stamp: 'JCB 7/18/2009 15:19'!
editPage: aStructure
	pageEdit isNil
		ifTrue: [pageEdit := PREditCommand new write: 'change the title' using: PRStructure descriptionTitle.
			pageEdit setContext: self instance context.
			self instance context command: pageEdit.
			self instance context structure: aStructure].
	pageEdit execute! !

!PRAdvancedSearchWditgetTest methodsFor: 'accessing' stamp: 'JCB 7/18/2009 15:12'!
instance
	advancedSearch isNil
		ifTrue: [advancedSearch := self actualClass new onChangeContext: self contextInstance;
						 yourself].
	^ advancedSearch! !

!PRAdvancedSearchWditgetTest methodsFor: 'accessing' stamp: 'JCB 7/18/2009 15:06'!
root
	^ self instance context kernel root! !

!PRAdvancedSearchWditgetTest methodsFor: 'tests' stamp: 'JCB 7/18/2009 15:25'!
testSort
	"self debug: #testSort"
	self assert: (self instance sort: #()) isEmpty.
	self instance context kernel persistency: PRHistoryPersistency new.
	self editPage: self root.
	self assert: (self instance sort: self root enumerator everything contents reverse) first = self root! !

TestCase subclass: #PRMatchCriteriaTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-SW2-AdvancedSearch'!

!PRMatchCriteriaTest commentStamp: 'JCB 3/31/2009 18:33' prior: 0!
A PRMatchCriteriaTest is for testing a single criteria.

Instance Variables
!

!PRMatchCriteriaTest methodsFor: 'tests' stamp: 'JCB 4/4/2009 14:01'!
testAdvancedSearch
	"self debug: #testAdvancedSearch"
	| aPage criterions |
	(aPage := PRPage named: 'Root Page') contents: 'This contains a WORD that can be SEARCH'.
	criterions := OrderedCollection with: (PRMatchCriteria newFieldName: 'Title' textToMatch: 'Root' isNegated: false combinationType: #or isCaseSensitive: true).
	self assert: (aPage fullAdvancedSearch criterions: criterions) contents size = 1.
	criterions add: (PRMatchCriteria newFieldName: 'Contents' textToMatch: 'a word' isNegated: false combinationType: #and isCaseSensitive: true).
	self assert: (aPage fullAdvancedSearch criterions: criterions) contents size = 0.
	criterions last isCaseSensitive: false.
	self assert: (aPage fullAdvancedSearch criterions: criterions) contents size = 1.
	criterions add: (PRMatchCriteria newFieldName: '*' textToMatch: 'Can not find' isNegated: false combinationType: #or isCaseSensitive: false).
	self assert: (aPage fullAdvancedSearch criterions: criterions) contents size = 1.
	criterions last combinationType: #and.
	self assert: (aPage fullAdvancedSearch criterions: criterions) contents size = 0	
		! !

!PRMatchCriteriaTest methodsFor: 'tests' stamp: 'JCB 3/30/2009 22:47'!
testMatch
	"Test fields, negation, and case sensitivity
	self debug: #testMatch"
	| aPage search |
	(aPage := PRPage named: 'Root Page') contents: 'This contains a WORD that can be SEARCH'.
	search := PRMatchCriteria newFieldName: 'Title' textToMatch: 'Root Page' isNegated: false combinationType: #or isCaseSensitive: true.
	self assert: (search matchAgainst: aPage).
	search isNegated: true.
	self deny: (search matchAgainst: aPage).
	search isCaseSensitive: false.
	self deny: (search matchAgainst: aPage).
	search textToMatch: 'root page'.
	self deny: (search matchAgainst: aPage).
	search isCaseSensitive: true.
	self assert: (search matchAgainst: aPage).
	search fieldName: 'Contents'; isNegated: false.
	self deny: (search matchAgainst: aPage).
	search fieldName: '*'; isCaseSensitive: false.
	self assert: (search matchAgainst: aPage).
	
! !
